#' Package matrixStats
#'
#' High-performing functions operating on rows and columns of matrices, e.g.
#' col / rowMedians(), col / rowRanks(), and col / rowSds().  Functions
#' optimized per data type and for subsetted calculations such that both memory
#' usage and processing time is minimized.  There are also optimized
#' vector-based methods, e.g. binMeans(), madDiff() and weightedMedian().
#'
#' @section How to cite this package:
#' Henrik Bengtsson (2017). matrixStats: Functions that Apply to Rows and
#' Columns of Matrices (and to Vectors). R package version 0.52.2.
#' https://github.com/HenrikBengtsson/matrixStats
#'
#' @author Henrik Bengtsson, Hector Corrada Bravo, Robert Gentleman, Ola
#' Hossjer, Harris Jaffee, Dongcan Jiang, Peter Langfelder
#'
#' @keywords package
#'
#' @name matrixStats-package
#' @aliases matrixStats
"_PACKAGE"
