% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/999.package.R
\docType{package}
\name{matrixStats-package}
\alias{matrixStats-package}
\alias{matrixStats}
\title{Package matrixStats}
\description{
High-performing functions operating on rows and columns of matrices, e.g.
col / rowMedians(), col / rowRanks(), and col / rowSds().  Functions
optimized per data type and for subsetted calculations such that both memory
usage and processing time is minimized.  There are also optimized
vector-based methods, e.g. binMeans(), madDiff() and weightedMedian().
}
\section{How to cite this package}{

Henrik Bengtsson (2017). matrixStats: Functions that Apply to Rows and
Columns of Matrices (and to Vectors). R package version 0.52.2.
https://github.com/HenrikBengtsson/matrixStats
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/HenrikBengtsson/matrixStats}
  \item Report bugs at \url{https://github.com/HenrikBengtsson/matrixStats/issues}
}

}
\author{
Henrik Bengtsson, Hector Corrada Bravo, Robert Gentleman, Ola
Hossjer, Harris Jaffee, Dongcan Jiang, Peter Langfelder
}
\keyword{package}
